          SECTION I                                                FILE MORE.I
                                    WRITING PROGRAMS
                                    ****************

              Writing computer programs is an intellectually stimulating
          occupation which allows you to express yourself in many ways. You
          soon find that the part of the program which actually does the work
          is small compared to the remainder of the program which hopefully
          makes the program a joy to use, or to employ the popular term,
          'user-friendly'. Thus you should always endeavour to provide a well
          designed screen which is pleasing to the eye and  easy to
          understand. You should make the program easy to use and ensure that
          the operator is never left guessing as to precisely what input is
          required. It may be acceptable in a program designed purely for your
          own use to use cryptic phrases or even just question marks as
          prompts but another user deserves better. If the operator makes an
          incorrect response to a prompt- e.g enters a word instead of a
          number, you should arrange for a warning message to appear on screen
          and another opportunity to enter the data correctly.
              Finding a subject for a computer program is not always easy, but
          as with writing a story you should examine your own fields of
          experience first. If you are a teacher of a foreign language you
          might consider a language course, complete with exercises, and
          tables of verbs.
              An engineer, scientist or mathematician could have fun with
          formulae, whilst a builder could write a program to assist with
          estimates or quotations.  But whatever you do, do it with
          imagination and try to avoid the obvious. Write the program that you
          have always needed and you could find that other people have also
          been waiting for such a program.
              Another source of program material is games. The PCW is not
          ideal for arcade-type alien-zapping games as it is too sedate, but
          you must be aware that programs are already available for chess,
          bridge, backgammon etc, together with text adventure programs. You
          could try translating card games, dice games or board games into
          programs and there are also outdoor pursuits such as skiing, sailing
          and other sports all of which could provide a challenge to produce a
          really fascinating program. Once you start thinking in these terms
          the list is endless.But how to start?
              Programming does not have to be confined to your computer
          keyboard. Choosing a subject for a program, and deciding how to
          present it can take several days, turning the problem over in your
          mind and scribbling on scraps of paper. Imagine the program as it is
          to unfold to the user; will you use menus?  How should the screen
          look?. Make notes as you scheme or some good ideas will be lost.You
          have to consider your  limitations as well as those of the machine
          but don't be too conservative; set yourself a challenge. You can
          always backtrack a little if you find that your target is too
          ambitious.
              When you have decided in principle how you would like the
          program to develop it is time to start practical programming. Having
          decided on a suitable and descriptive name for the program file
          write the first four lines.

          1 GOTO 4
          2 PRINT "SAVEing *** MEGAPROG.BAS *** : SAVE "A:MEGAPROG",A
          3 STOP
          4 REM

          You can then get on with writing your program knowing that at any
          time you can type GOTO 2 and the embryo program will be saved to
          disc at the same time as the message SAVEing *** MEGAPROG.BAS
          appears on screen. (the A: before MEGAPROG saves the program to
          drive A: and the ,A after saves the program in ASCII format). This
          is a great timesaver and will save a lot of hassle. You don't even
          have to remember the name of the program.   On the other hand the
          command RUN will run the program by leapfrogging lines 2 and 3.
              The next part of the program to write is the 'initialisation'
          (dreadful word); where you define your main variables, codes etc.
          such as 10 cls$=CHR$(27)+"E"+CHR$(27)+"H" -- which I am sure that
          you will recognise as the command for clearing the screen. A
          timesaver is to write a standard 'module' containing all your
          favourite codings and to save it under a filename such as INIT.BAS
          for use as required. I reserve lines 10 to 100 for this.

          You could also write a standard module for subroutines such as
          "Press [RETURN] to continue", particularly if you intend to write
          several similar programs. I usually start this module at a fairly
          high line number and the first line of this module should be

          50000 END     -- This will protect the subroutines from being read
          by accident. Examples of these two modules can be seen in the file
          TUTOR.BAS

          Now to the program proper. Break the program down into manageable
          tasks which can be tackled individually. For example, if you were
          writing a simulation of the card game 'PONTOON' , the tasks may be
          as follows :-

          1) Make array of all 52 cards
          2) Assign values to cards e.g Court cards =10, Aces =1 or 10 etc.
          3) Dealer deals 2 cards to himself and states total value
          4) Dealer deals 2 cards to player and states total value
          5) Player indicates 'Stick or twist". Dealer deals as required and
             states new value until turn is complete.
          6) Dealer then deals a further card or cards to himself until the
             game is resolved.
          7) NOTE : take into account that Ace can be 10 or 1

          These are the main tasks which are the bare bones of the game. You
          would be faced with many small problems within these tasks and also
          you would have to decide how to indicate the cards, screen layout
          etc., as well as making the program as foolproof and bug- free as
          possible. You may decide to compromise on these latter tasks, and
          whilst helping a player who makes a typing error,ignore one who is
          an absolute idiot.

          You may remember that a PONTOON game was included as light relief in
          the INTRO section. You can see how I solved some of the problems,
          and would perhaps like to improve the game, maybe by adding a
          betting element or a continuous points score, or by improving the
          presentation?. The other games are also worth studying and noting
          how various problems have been overcome. For example, in BIZZBUZZ I
          wanted to introduce a timed element. As BASIC can only do one job at
          a time I had to compromise and announce 'out of time' after the
          event instead of during it. See if you can discover what I mean.

          In writing parts of a program don't be afraid to separate the
          individual parts by a generous number. Put main parts 1000 lines
          apart. You can always renumber when the program is complete if you
          wish. Remember that you have upwards of 65000 lines, but leave some
          lines at the end as a scratchpad. By this I mean that you will
          sometimes want to experiment with a few lines to try a particular
          effect before committing yourself to using it in the program. If you
          achieve your desired effect you can renumber the lines to fit into
          your program: if not they can be deleted.

          Wherever possible seize any chance to cut down on typing!. If you
          find that several lines are similar then use EDIT or the left cursor
          key to call up a line for editing, being sure always to CHANGE THE
          LINE NUMBER FIRST so as to avoid changing the original line. AUTO is
          most useful too in saving time and typing. When writing this course
          I found that several near identical blocks of lines were required
          1000 or 2000 lines apart. By typing the command AUTO 2000 I was able
          to call up line 2000 and change it to line 3000. Pressing [RETURN]
          brought up line 2010 which I would change to 3010 (and edit if
          necessary). In this way I could get near identical blocks from lines
          2000 and 3000 which I could repeat for 4000,5000 etc.  AUTO was also
          of great use when subsequent editing of similar lines spaced at
          equal intervals was needed.  AUTO 2000,1000 brought up line 2000,
          then 3000,4000,5000 in succession for rapid editing.

          Another use for AUTO is in aligning lines in menus etc.where unlike
          the EDIT command it brings up lines directly above each other,
          making the task much easier.

          I hope from the foregoing that you will see the sense in keeping to
          equal and logical line spacings. A little forethought here can save
          a lot of time, not least in editing and debugging.

          Don't be mean with REM statements. It is easy to put in relevant
          notes whilst writing a program; a week later you'll be glad that you
          did! In particular, note where GOTOs and GOSUBS; e.g
                  230 GOTO 100 : REM ...  To main menu
                  240 gosub 40300 : REM Sub/r options menu
                  250 GOSUB 40350 : REM Sub/r 'Section titles',  then label
          the start of the subroutines with the same title. This will save a
          lot of time. paticularly in a large program. Superfluous REMs can be
          edited out on completion of the program, but keep one unabridged
          copy for possible use later. Rem notes don't affectmthe speed of
          working of the program, but do use up memory.

          SUMMARY   Just like the author of a book, keep your programs within
          =======   your own field of experience.  Being an engineer, my first
          program was about the stressing and deflection of beams. I used it
          nearly every day, and it saved a tremendous amount of work with a
          calculator, yet as a program it was simple, even crude.

          Get your program working first, even though not perfectly, then
          start to add the 'user-friendly' bits, but use REMs right from the
          start, and make daily notes on paper, otherwise it will seem like
          pushing an ever-increasing load uphill. From time to time, review
          the whole program and assess what remains to be done.

          Don't be afraid to leave plenty of room between parts of a program,
          and use regular spacings, 1000,2000,3000 etc. between similar
          modules to simplify copying, editing, debugging etc.

          Try to do a little each day; if you can't get on the computer
          consult LLISTED copy and notes then do a little programming on
          paper. It is very easy to lose the thread of a program and it takes
          time to backtrack. When the program is complete and fully annotated,
          leave it for a week or so, then run it. You will be pleased with it,
          I promise, but will probably find some bugs or niggly details that
          need attending to. Some people say that a computer program, like a
          book, is never finished, but you must learn to draw a line under
          each program, then go on to the next.

          Finally, save your program regularly as it progresses, and take a
          back-up copy on another disc. If you don't, sooner or later you'll
          be sorry.

          End of file MORE.I
   